@@ -1,7 +1,5 @@
 const {BrowserWindow, app, dialog, ipcMain} = require('electron')
 const path = require('path')
-const fs = require('fs')
-const {statSyncNoException} = require('fs-plus')
 const url = require('url')
 const {EventEmitter} = require('events')
 
@@ -53,7 +51,9 @@ class AtomWindow extends EventEmitter {
     if (this.shouldAddCustomTitleBar()) options.titleBarStyle = 'hidden'
     if (this.shouldAddCustomInsetTitleBar()) options.titleBarStyle = 'hiddenInset'
     if (this.shouldHideTitleBar()) options.frame = false
-    this.browserWindow = new BrowserWindow(options)
+
+    const BrowserWindowConstructor = settings.browserWindowConstructor || BrowserWindow
+    this.browserWindow = new BrowserWindowConstructor(options)
 
     Object.defineProperty(this.browserWindow, 'loadSettingsJSON', {
       get: () => JSON.stringify(Object.assign({
@@ -73,8 +73,11 @@ class AtomWindow extends EventEmitter {
     if (this.loadSettings.safeMode == null) this.loadSettings.safeMode = false
     if (this.loadSettings.clearWindowState == null) this.loadSettings.clearWindowState = false
 
-    this.loadSettings.initialPaths = locationsToOpen.map(location => location.pathToOpen).filter(Boolean)
-    this.loadSettings.initialPaths.sort()
+    this.addLocationsToOpen(locationsToOpen)
+
+    this.loadSettings.hasOpenFiles = locationsToOpen
+      .some(location => location.pathToOpen && !location.isDirectory)
+    this.loadSettings.initialProjectRoots = this.projectRoots
 
     // Only send to the first non-spec window created
     if (includeShellLoadTime && !this.isSpec) {
@@ -84,7 +87,6 @@ class AtomWindow extends EventEmitter {
       }
     }
 
-    this.representedDirectoryPaths = this.loadSettings.initialPaths
     if (!this.loadSettings.env) this.env = this.loadSettings.env
 
     this.browserWindow.on('window:loaded', () => {
@@ -121,8 +123,8 @@ class AtomWindow extends EventEmitter {
     if (hasPathToOpen && !this.isSpecWindow()) this.openLocations(locationsToOpen)
   }
 
-  hasProjectPath () {
-    return this.representedDirectoryPaths.length > 0
+  hasProjectPaths () {
+    return this.projectRoots.length > 0
   }
 
   setupContextMenu () {
@@ -133,18 +135,20 @@ class AtomWindow extends EventEmitter {
     })
   }
 
-  containsPaths (paths) {
-    return paths.every(p => this.containsPath(p))
+  containsLocations (locations) {
+    return locations.every(location => this.containsLocation(location))
   }
 
-  containsPath (pathToCheck) {
-    if (!pathToCheck) return false
-    let stat
-    return this.representedDirectoryPaths.some(projectPath => {
-      if (pathToCheck === projectPath) return true
-      if (!pathToCheck.startsWith(path.join(projectPath, path.sep))) return false
-      if (stat === undefined) stat = statSyncNoException(pathToCheck)
-      return !stat || !stat.isDirectory()
+  containsLocation (location) {
+    if (!location.pathToOpen) return false
+
+    return this.projectRoots.some(projectPath => {
+      if (location.pathToOpen === projectPath) return true
+      if (location.pathToOpen.startsWith(path.join(projectPath, path.sep))) {
+        if (!location.exists) return true
+        if (!location.isDirectory) return true
+      }
+      return false
     })
   }
 
@@ -234,6 +238,7 @@ class AtomWindow extends EventEmitter {
   }
 
   async openLocations (locationsToOpen) {
+    this.addLocationsToOpen(locationsToOpen)
     await this.loadedPromise
     this.sendMessage('open-locations', locationsToOpen)
   }
@@ -246,6 +251,18 @@ class AtomWindow extends EventEmitter {
     this.sendMessage('did-fail-to-read-user-settings', message)
   }
 
+  addLocationsToOpen (locationsToOpen) {
+    const roots = new Set(this.projectRoots || [])
+    for (const {pathToOpen, isDirectory} of locationsToOpen) {
+      if (isDirectory) {
+        roots.add(pathToOpen)
+      }
+    }
+
+    this.projectRoots = Array.from(roots)
+    this.projectRoots.sort()
+  }
+
   replaceEnvironment (env) {
     this.browserWindow.webContents.send('environment', env)
   }
@@ -378,7 +395,7 @@ class AtomWindow extends EventEmitter {
   showSaveDialog (options, callback) {
     options = Object.assign({
       title: 'Save File',
-      defaultPath: this.representedDirectoryPaths[0]
+      defaultPath: this.projectRoots[0]
     }, options)
 
     if (typeof callback === 'function') {
@@ -410,10 +427,10 @@ class AtomWindow extends EventEmitter {
     return this.browserWindow.setRepresentedFilename(representedFilename)
   }
 
-  setRepresentedDirectoryPaths (representedDirectoryPaths) {
-    this.representedDirectoryPaths = representedDirectoryPaths
-    this.representedDirectoryPaths.sort()
-    this.loadSettings.initialPaths = this.representedDirectoryPaths
+  setProjectRoots (projectRootPaths) {
+    this.projectRoots = projectRootPaths
+    this.projectRoots.sort()
+    this.loadSettings.initialProjectRoots = this.projectRoots
     return this.atomApplication.saveCurrentWindowOptions()
   }
 
@@ -428,4 +445,8 @@ class AtomWindow extends EventEmitter {
   disableZoom () {
     return this.browserWindow.webContents.setVisualZoomLevelLimits(1, 1)
   }
+
+  getLoadedPromise () {
+    return this.loadedPromise
+  }
 }
